package net.wanji.opt.common;

import net.wanji.common.gts.Tools;
import net.wanji.opt.common.enums.TimeGranularityEnum;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.filefilter.FileFilterUtils;
import org.apache.commons.io.filefilter.IOFileFilter;
import org.joda.time.*;
import org.joda.time.format.DateTimeFormat;

import java.io.File;
import java.util.*;
import java.util.stream.Collectors;

public class EsDateIndexUtil {

    public static final String YMDHM_FORMATTER = "yyyyMMddHHmm";
    public static final String YMDHMSFORMATTER = "yyyyMMddHHmmss";
    public static final String YMD_HM_FORMATTER = "yyyy-MM-dd HH:mm:ss";
    public static final String YMD_H_FORMATTER = "yyyyMMddHH";
    public static final String YMD_FORMATTER = "yyyyMMdd";
    public static final String YM_FORMATTER = "yyyyMM";
    public static final String HMS_FORMATTER = "HH:mm:ss";
    public static final String HMS_SSS_FORMATTER = "HH:mm:ss SSS";
    public static final String HM_FORMATTER = "HH:mm";
    public static final String H_FORMATTER = "HH:00";

    private static final String FORMATSTR = "%s_%s";

    /**
     * 获取时段范围内不同时间粒度的时间轴
     * @param groupType 0：5分钟 1：15分钟 2：30分钟 3：1小时 4：天粒度
     * @param startTime 开始时间
     * @param endTime 截止时间
     * @return
     */
    public static  Set<String> getTimeGranularityAxis(String groupType,String startTime, String endTime) {
        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        //===========================根据开始、结束时间输出完整时刻点=================================================//
        DateTime start = DateTime.parse(startTime, DateTimeFormat.forPattern(EsDateIndexUtil.YMD_HM_FORMATTER));
        DateTime end = DateTime.parse(endTime, DateTimeFormat.forPattern(EsDateIndexUtil.YMD_HM_FORMATTER));
        if (Objects.equals("0", groupType)) {
            sortedSet.addAll(EsDateIndexUtil.getTimeScopeList(start, end, TimeGranularityEnum.FIVE_MINUTE, EsDateIndexUtil.HM_FORMATTER));
        } else if (Objects.equals("1", groupType)) {
            sortedSet.addAll(EsDateIndexUtil.getTimeScopeList(start, end, TimeGranularityEnum.FIFTEEN_MINUTE, EsDateIndexUtil.HM_FORMATTER));
        } else if (Objects.equals("2", groupType)) {
            sortedSet.addAll(EsDateIndexUtil.getTimeScopeList(start, end, TimeGranularityEnum.THIRTY_MINUTE, EsDateIndexUtil.HM_FORMATTER));
        } else if (Objects.equals("3", groupType)) {
            sortedSet.addAll(EsDateIndexUtil.getTimeScopeList(start, end, TimeGranularityEnum.ONE_HOUR, EsDateIndexUtil.H_FORMATTER));
        } else if (Objects.equals("4", groupType)) {
            sortedSet.addAll(EsDateIndexUtil.getTimeScopeList(start, end, TimeGranularityEnum.ONE_DAY, EsDateIndexUtil.YMD_FORMATTER));
        } else if (Objects.equals("5", groupType)) {
            sortedSet.addAll(EsDateIndexUtil.getTimeScopeList(start, end, TimeGranularityEnum.TEN_MINUTE, EsDateIndexUtil.HM_FORMATTER));
        }
        return sortedSet;
    }

    /**
     * 根据开始结束时间获取不同时间粒度的时刻点
     *
     * @param start 开始时间戳
     * @param end   截止时间戳
     * @param timeGranularityEnum  时间枚举类型
     * @return
     */
    public static List<String> getTimeScopeList( DateTime start, DateTime end, TimeGranularityEnum timeGranularityEnum,String dateFormat) {
        List<String> indexList = new ArrayList<>();

        int diffNum = Minutes.minutesBetween(start, end).getMinutes();

        int unitTime = 0;
        String DATE_FORMAT  = YMD_HM_FORMATTER;
        if (Objects.equals(TimeGranularityEnum.FIVE_MINUTE, timeGranularityEnum)) {
            unitTime = 5;
        }else if (Objects.equals(TimeGranularityEnum.FIFTEEN_MINUTE, timeGranularityEnum)) {
            unitTime = 15;
        }else if (Objects.equals(TimeGranularityEnum.THIRTY_MINUTE, timeGranularityEnum)) {
            unitTime = 30;
        }else if (Objects.equals(TimeGranularityEnum.ONE_HOUR, timeGranularityEnum)) {
            unitTime = 60;
        }else if (Objects.equals(TimeGranularityEnum.ONE_DAY, timeGranularityEnum)) {
            unitTime = 24*60;
        }else if (Objects.equals(TimeGranularityEnum.TEN_MINUTE, timeGranularityEnum)) {
            unitTime = 10;
        }

        for (int i = 0; i <= diffNum; i=i+unitTime) {
            String format = start.plusMinutes(i).toString(dateFormat);
            indexList.add(format);
        }
        return indexList;
    }


    /**
     * 根据开始结束时间匹配对应索引
     *
     * @param indexName 索引前缀
     * @param startTime 开始时间戳
     * @param endTime   截止时间戳
     * @param timeType  1:分钟 2：小时 3：天 4：月
     * @return
     */
    public static String[] getIndexByTime(String indexName, long startTime, long endTime, String timeType) {
        List<String> indexList = new ArrayList<>();

        DateTime start = new DateTime(startTime);
        DateTime end = new DateTime(endTime);
        int diffNum = 0;
        //push单位秒
        int unitTime = 0;
        String DATE_FORMAT = null;
        if (Objects.equals("1", timeType)) {
            DATE_FORMAT = YMDHM_FORMATTER;
            diffNum = Minutes.minutesBetween(start, end).getMinutes();
            unitTime = 60;
        } else if (Objects.equals("2", timeType)) {
            DATE_FORMAT = YMD_H_FORMATTER;
            diffNum = Hours.hoursBetween(start, end).getHours();
            unitTime = 60 * 60;
        } else if (Objects.equals("3", timeType)) {
            DATE_FORMAT = YMD_FORMATTER;
            diffNum = Days.daysBetween(start, end).getDays();
            unitTime = 24 * 60 * 60;
        } else if (Objects.equals("4", timeType)) {
            DATE_FORMAT = YM_FORMATTER;
            diffNum = Months.monthsBetween(start, end).getMonths();
            unitTime = 30 * 24 * 60 * 60;
        }

        for (int i = 0; i <= diffNum; i++) {
            String format = String.format(FORMATSTR, indexName, start.plusSeconds(i * unitTime).toString(DATE_FORMAT));
            indexList.add(format);
        }
        return indexList.toArray(new String[0]);
    }



    /**
     * 根据开始结束时间匹配 文件完整路径
     *
     * @param fileDirectory 文件目录
     * @param startTime 开始时间戳
     * @param endTime   截止时间戳
     * @param timeType  1:分钟 2：小时 3：天 4：月 5；10分钟
     * @return
     */
    public static String[] getFileListByTime(String fileDirectory, String filePrefix,long startTime, long endTime, String timeType) {
        Set<String> indexList = new TreeSet<>();

        ///data/track_partitions/dt=2023-08-09/ht=11/mt=00

        DateTime start = new DateTime(startTime);
        DateTime end = new DateTime(endTime);

        while (start.isBefore(end)){


            String dt = start.toString("yyyy-MM-dd");
            dt = "dt="+dt;

            int hour = start.getHourOfDay();
            String ht = Tools.fillZeroFront(hour+"",2);
            ht = "ht="+ht;

            int minute = start.getMinuteOfHour();
            int index = minute/10*10;
            String mt = Tools.fillZeroFront(index+"",2);
            mt = "mt="+mt;

            String directory = fileDirectory +"/"+dt+"/"+ht+"/"+mt;
            if (!new File(directory).exists()){
                start = start.plusMinutes(1);
                continue;
            }


            IOFileFilter ioFileFilter = FileFilterUtils.prefixFileFilter(filePrefix);

             List<File> fileList = FileUtils.listFiles(new File(directory),ioFileFilter,null).stream().collect(Collectors.toList());

            String fileName = null;
            for (File dataFile : fileList) {
                fileName = dataFile.getName();
                String filePath = directory +"/"+fileName;
                if(!indexList.contains(filePath)) {
                    indexList.add(filePath);
                }
            }
            start = start.plusMinutes(1);
//            System.out.println("start:"+start.toString(DateUtil.DATE_FORMAT));
        }
//        System.out.println("end:"+start.toString(DateUtil.DATE_FORMAT));
//        System.out.println(indexList.toString());
        // indexList = indexList.stream().sorted().collect(Collectors.toSet());
        return indexList.toArray(new String[0]);
    }

    /**
     * 获取当前时间实时索引
     *
     * @param indexName
     * @return
     */
//    public static String[] getIndexByNow(String indexName) {
//        long start = DateTime.now().plusMonths(-1).withTimeAtStartOfDay().getMillis();
//        long end = DateTime.now().getMillis();
//        return getIndexByTime(indexName, start, end);
//    }
//
//    public static String[] getIndexBySingle(String indexName, long time) {
//        long start = new DateTime(time).plusMonths(-1).withTimeAtStartOfDay().getMillis();
//        return getIndexByTime(indexName, start, time);
//    }
    public static String getInsertIndex(String indexName) {
        return String.format("%s_%s", indexName, DateTime.now().toString(YM_FORMATTER));
    }


    public static List<String> getTimeList(String formatter, long interval, long startTime, long endTime) {
        List<String> timeList = new ArrayList<>();
        for (long i = startTime; i < endTime; i += interval) {
            timeList.add(new DateTime(i).toString(formatter));
        }
        return timeList;
    }


    public static List<String> getMinutesStr() {
        List<String> timeStr = new ArrayList<>();
        for (int i = 0; i < 24; i++) {
            String h;
            if (i < 10) {
                h = "0" + i;
            } else {
                h = i + "";
            }
            for (int j = 0; j < 60; j++) {
                String m;
                if (j < 10) {
                    m = "0" + j;
                } else {
                    m = j + "";
                }
                timeStr.add(h + ":" + m);
            }
        }
        return timeStr;
    }

    public static void main(String[] args) {
        DateTime start  =  DateTime.now().withTimeAtStartOfDay();
        DateTime end  =   DateTime.now();
       List<String> indexByTime = EsDateIndexUtil.getTimeScopeList(start, end, TimeGranularityEnum.ONE_HOUR,"HH:mm");

        System.out.println(indexByTime);

    }


}
