package net.wanji.opt.controller;

import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dto.EventInfoTrafficStatusDTO;
import net.wanji.databus.vo.EventInfoTrafficStatusVO;
import net.wanji.opt.entity.EventAlarmInfo;
import net.wanji.opt.po.base.EventStatisticPo;
import net.wanji.opt.service.EventService;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;


@RestController
@Slf4j
@RequestMapping("/event")
@Api(value="EventController", description="事件相关", tags = "事件相关")
public class EventController {

    @Resource
    EventService eventService;


    @GetMapping("/count")
    @ApiOperation(httpMethod="GET",value="事件总次数统计", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossId", value = "路口ID", required = true, dataType = "String",defaultValue = "13N8J0B5P10"),
            @ApiImplicitParam(name = "eventType", value = "事件类型 701:空放 702：失衡 703：溢出", required = true, dataType = "String",defaultValue = "701"),
            @ApiImplicitParam(name = "startTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = true, dataType = "String",defaultValue = "2025-01-05 00:00:00"),
            @ApiImplicitParam(name = "endTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = true, dataType = "String",defaultValue = "2025-01-05 23:59:59"),

    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = EventStatisticPo.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject evaluateCrossDetail(String crossId,String eventType,String startTime,String endTime) {
        EventStatisticPo result = null;
        try {
            result = eventService.findCrossEventCount(crossId,eventType,startTime,endTime );
            return JsonViewObject.newInstance().success(result);
        } catch (DubboProviderException e) {
            e.printStackTrace();
            return JsonViewObject.newInstance().fail(e);
        }

    }

    @GetMapping("/distribute")
    @ApiOperation(httpMethod="GET",value="空放、失衡、溢出事件分布情况", notes="")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossId", value = "路口ID", required = true, dataType = "String",defaultValue = "13N8J0B5P10"),
            // @ApiImplicitParam(name = "eventType", value = "事件类型 701:空放 702：失衡 703：溢出", required = false, dataType = "String",defaultValue = "701"),
            @ApiImplicitParam(name = "startTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = true, dataType = "String",defaultValue = "2025-01-05 00:00:00"),
            @ApiImplicitParam(name = "endTime", value = "截止时间，格式：yyyy-MM-dd HH:mm:ss", required = true, dataType = "String",defaultValue = "2025-01-05 23:59:59"),

    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = EventStatisticPo.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject distribute(String crossId,String startTime,String endTime) {

        try {
            List<Map<String,Object>> result = eventService.findEventDistribute(crossId,startTime,endTime );
            return JsonViewObject.newInstance().success(result);
        } catch (DubboProviderException e) {
            e.printStackTrace();
            return JsonViewObject.newInstance().fail(e);
        }

    }

    @PostMapping("/trafficStatus")
    @ApiOperation(value="路口监测、干线监测-交通状态", notes="")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = EventStatisticPo.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject trafficStatus(@RequestBody @Valid EventInfoTrafficStatusDTO eventInfoTrafficStatusDTO) {

        try {
            EventInfoTrafficStatusVO result = eventService.trafficStatus(eventInfoTrafficStatusDTO );
            return JsonViewObject.newInstance().success(result);
        } catch (DubboProviderException e) {
            e.printStackTrace();
            return JsonViewObject.newInstance().fail(e);
        }

    }


    @GetMapping("/getAlarmInfo")
    @ApiOperation(hidden = false, value = "实时/历史事件报警获取", notes = "", produces = "application/json")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "realtimeType", value = "查询类型：1为实时 2为历史", required = false, dataType = "String"),
            @ApiImplicitParam(name = "eventType", value = "事件类型，701：空放 702：失衡 703：溢出 705：干线缓行 706：干线拥堵 707：路口拥堵", required = false, dataType = "String"),
            @ApiImplicitParam(name = "name", value = "按名称模糊查询", required = false, dataType = "String"),

    }
    )
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = EventAlarmInfo.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject list(@RequestParam(defaultValue = "") String realtimeType, @RequestParam(defaultValue = "") String eventType , @RequestParam(defaultValue = "") String name) {
        try {
            JsonViewObject jsonViewObject = JsonViewObject.newInstance();
            List<EventAlarmInfo> eventAlarmInfo = eventService.findEventAlarmInfo(realtimeType,eventType,name);
            return jsonViewObject.success(eventAlarmInfo);
        } catch (Exception e) {
            log.error("", e);
            return JsonViewObject.newInstance().fail("请求失败");
        }

    }

    @GetMapping("/getCrossCurrentStatus")
    @ApiOperation(hidden = false, value = "路口实时状态接口", notes = "", produces = "application/json")
    @ApiImplicitParams({
    }
    )
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = EventAlarmInfo.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject getCrossNowStatus() {
        try {
            JsonViewObject jsonViewObject = JsonViewObject.newInstance();
            List<EventAlarmInfo> eventAlarmInfo = eventService.findNotFinishAlarmInfo();
            return jsonViewObject.success(eventAlarmInfo);
        } catch (Exception e) {
            log.error("", e);
            return JsonViewObject.newInstance().fail("请求失败");
        }

    }

}