package net.wanji.opt.controller.induce;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.i18n.I18nResourceBundle;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.framework.rest.ValidationGroups;
import net.wanji.opt.entity.InduceFontHist;
import net.wanji.opt.service.induce.InduceFontHistService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.ws.rs.core.MediaType;
import java.util.List;

/**
 * @author duanruiming
 * @date 2025/01/21 14:37
 */
@Api(value = "InduceFontHistController", description = "诱导屏-文字下发历史信息")
@RestController
@RequestMapping("/induceFontHist")
@Slf4j
public class InduceFontHistController {
    @Resource
    private InduceFontHistService induceFontHistService;

    @ApiOperation(value = "获取所有诱导文字下发记录", notes = "获取所有诱导文字下发记录",
            response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/byAll", produces = MediaType.APPLICATION_JSON)
    public JsonViewObject getAll(@RequestParam(defaultValue = "") String equipCode) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            List<InduceFontHist> list = induceFontHistService.getAll(equipCode);
            jsonViewObject.success(list);
        } catch (Exception e) {
            jsonViewObject.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonViewObject;
    }


    @ApiOperation(value = "诱导屏文字编辑新增修改", notes = "诱导屏文字编辑新增修改",
            response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/editor", produces = MediaType.APPLICATION_JSON,
            consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject editor(@ApiParam(value = "查询条件", required = true)
                                 @RequestBody @Validated({ValidationGroups.Query.class})
                                 InduceFontHist induceFontHist) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            induceFontHistService.editor(induceFontHist);
            jsonViewObject.success("编辑成功");
        } catch (Exception e) {
            jsonViewObject.fail(I18nResourceBundle.getConstants("POST_FAILED_MSG"));
            log.error("{} editor error", this.getClass().getSimpleName(), e);
        }
        return jsonViewObject;
    }

    @ApiOperation(value = "通过编号获取诱导屏最新文字", notes = "通过编号获取诱导屏最新文字",
            response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/byLastOne", produces = MediaType.APPLICATION_JSON)
    public JsonViewObject getLastOne(@RequestParam("equipCode") String equipCode) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            InduceFontHist induceFontHist = induceFontHistService.getLastOne(equipCode);
            jsonViewObject.success(induceFontHist);
        } catch (Exception e) {
            jsonViewObject.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} byLastOne error", this.getClass().getSimpleName(), e);
        }
        return jsonViewObject;
    }

    @ApiOperation(value = "通过字体id删除诱导屏字体信息", notes = "通过字体id删除诱导屏字体信息",
            response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/deleteOneById", produces = MediaType.APPLICATION_JSON)
    public JsonViewObject deleteOneById(@RequestParam("id") Integer id) {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            induceFontHistService.deleteOneById(id);
            jsonViewObject.success("删除成功");
        } catch (Exception e) {
            jsonViewObject.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} byLastOne error", this.getClass().getSimpleName(), e);
        }
        return jsonViewObject;
    }

}
