package net.wanji.opt.controller.signalopt;

import io.swagger.annotations.*;
import net.wanji.common.framework.i18n.I18nResourceBundle;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.po.CrossDataRealtimePO;
import net.wanji.databus.po.CrossDirDataRealtimePO;
import net.wanji.databus.po.CrossDirStatusDataPO;
import net.wanji.databus.po.TBaseCrossInfo;
import net.wanji.opt.dto.strategy.AddOrUpdateSceneDTO;
import net.wanji.opt.entity.GreenwaveHist;
import net.wanji.opt.po.trend.HoloEventInfoPO;
import net.wanji.opt.service.CrossIndexService;
import net.wanji.opt.vo.AIOptResultVO;
import net.wanji.opt.vo.CrossEventListPO;
import net.wanji.opt.vo.CrossOptResult;
import net.wanji.opt.vo.CrossStatusCountVO;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.ws.rs.core.MediaType;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author duanruiming
 * @date 2024/11/25 19:10
 */
@Api(value = "CrossIndexController", description = "路口指标控制器")
@RequestMapping("/crossIndex")
@RestController
public class CrossIndexController {

    @Resource
    private CrossIndexService crossIndexService;

    @ApiOperation(value = "路口信息查询", notes = "路口信息查询", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/crossInfoList")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = TBaseCrossInfo.class),
    })
    public JsonViewObject crossInfoList() {
        List<TBaseCrossInfo> baseCrossInfoPOS = crossIndexService.crossInfoList();
        return JsonViewObject.newInstance().success(baseCrossInfoPOS);
    }

    @ApiOperation(value = "路口方向指标", notes = "路口方向指标", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/crossDirIndex",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject crossDirIndex(@RequestBody CrossIdBO crossIdBO) {
        Map<Integer, CrossDirDataRealtimePO> result = crossIndexService.crossDirIndex(crossIdBO);
        return JsonViewObject.newInstance().success(result);
    }

    @ApiOperation(value = "路口指标", notes = "路口指标", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/crossIndex",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject crossIndex(@RequestBody CrossIdBO crossIdBO) {
        CrossDataRealtimePO crossDataRealtimePO = crossIndexService.crossIndex(crossIdBO);
        return JsonViewObject.newInstance().success(crossDataRealtimePO);
    }

    @ApiOperation(value = "方案优化曲线", notes = "优化监测-方案优化曲线", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/crossOptResultList",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject crossOptResultList(@RequestBody CrossIdBO crossIdBO) {
        List<CrossOptResult> results = null;
        try {
            results = crossIndexService.crossOptResultList(crossIdBO);
        } catch (Exception e) {
            JsonViewObject.newInstance().fail("方案优化曲线查询异常");
        }
        return JsonViewObject.newInstance().success(results);
    }


    @ApiOperation(value = "AI路口", notes = "优化监测-AI路口", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/crossAIList")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AIOptResultVO.class),
    })
    public JsonViewObject crossAIList() {
        List<AIOptResultVO> results = null;
        try {
            results = crossIndexService.crossAIList();
        } catch (Exception e) {
            JsonViewObject.newInstance().fail("优化监测-AI路口查询异常");
        }
        return JsonViewObject.newInstance().success(results);
    }

    @ApiOperation(value = "优化监测-问题诊断", notes = "优化监测-问题诊断", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/crossStatusCount")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossStatusCountVO.class),
    })
    public JsonViewObject crossStatusCount(String crossId) {
        List<CrossStatusCountVO> results = new ArrayList<>();
        try {
            results = crossIndexService.crossStatusCount(crossId);
        } catch (Exception e) {
            JsonViewObject.newInstance().fail("优化监测-问题诊断");
        }
        return JsonViewObject.newInstance().success(results);
    }

    @ApiOperation(value = "交通体检-路口监测-事件方向转向", notes = "交通体检-路口监测-事件方向转向", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/crossEventDirTurn")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = HoloEventInfoPO.class),
    })
    public JsonViewObject crossEventDirTurn(String crossId) {
        HoloEventInfoPO result = new HoloEventInfoPO();
        try {
            result = crossIndexService.crossEventDirTurn(crossId);
        } catch (Exception e) {
            JsonViewObject.newInstance().fail("优化监测-问题诊断");
        }
        return JsonViewObject.newInstance().success(result);
    }

    @ApiOperation(value = "查询各个方向交通状态变化趋势", notes = "查询各个方向交通状态变化趋势", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/crossDirStatusData")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject crossDirStatusData(@RequestBody CrossIdBO crossIdBO) {
        CrossDirStatusDataPO result = new CrossDirStatusDataPO();
        try {
            result = crossIndexService.selectByCrossIdAndHour(crossIdBO.getCrossId(),2);
        } catch (Exception e) {
            JsonViewObject.newInstance().fail("查询各个方向交通状态变化趋势");
        }
        return JsonViewObject.newInstance().success(result);
    }


    @GetMapping("/selectCrossEventList")
    @ApiOperation(httpMethod="GET",value="路口监测左下角路口事件列表", notes="")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功", response = GreenwaveHist.class,
                    responseHeaders = {@ResponseHeader(name = "Content-Type", description = "application/json")})
    })
    public JsonViewObject selectCrossEventList() {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        try {
            List<CrossEventListPO> list = crossIndexService.selectCrossEventList();
            jsonView.success(list);
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
        }

        return jsonView;
    }

}
