package net.wanji.opt.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.CrossStatusEnum;
import net.wanji.common.framework.Constants;
import net.wanji.databus.dao.mapper.CrossDataRealtimeMapper;
import net.wanji.databus.po.CrossDataRealtimePO;
import net.wanji.opt.kafka.ConsumerHandler;
import net.wanji.opt.service.CrossOptimizeService;
import net.wanji.opt.service.impl.LaneInfoServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @author hfx
 * @date 2023/1/16 16:39
 * @desc RealtimeOptimizeTask
 */

@RequiredArgsConstructor
@Component
@Slf4j
public class CrossMonitorTask {

    private final CrossOptimizeService crossOptimizeService;
    private final ConsumerHandler consumerHandler;
    private final CrossDataRealtimeMapper crossDataRealtimeMapper;
    private final LaneInfoServiceImpl laneInfoService;

    /**
     * 路口实时监测
     *
     */
    //@Scheduled(fixedRate = 1 * 60 * 1000)
    //public void realtimeMonitor() {
    //    log.info("路口实时监测任务...");
        // 获取路口实时监测数据
        //List<CrossDataRealtimePO> crossDataRealtimePOList = listCrossDataRealtime();
        //log.info("路口实时数据：{}条", crossDataRealtimePOList.size());
        // 测试时无数据暂时注释
//        if(crossDataRealtimePOList.isEmpty()) {
//            return ;
//        }

        // 获取所有信控路口异常数据（失衡/拥堵/溢出/死锁）
        //List<CrossDataRealtimePO> abnormalCrossList = listAbnormalCross(crossDataRealtimePOList);
        //log.info("加载异常路口：{}条", abnormalCrossList.size());
//        if(abnormalCrossList.isEmpty()) {
//            return ;
//        }

        // 调用路口实时优化
        //crossOptimizeService.realtimeOptimize(abnormalCrossList, crossDataRealtimePOList);

    //}

    /**
     * 获取异常路口数据
     *
     * @return
     */
    public List<CrossDataRealtimePO> listAbnormalCross(List<CrossDataRealtimePO> crossDataRealtimePOList) {

        List<CrossDataRealtimePO> dtoList = new ArrayList<>();
        CrossDataRealtimePO dto = null;
        int status = 0;
        for (CrossDataRealtimePO po : crossDataRealtimePOList) {

            // 获取失衡、拥堵、溢出路口数据
            if (Constants.SystemParam.NO.equals(po.getIsSpillover()) && Constants.SystemParam.NO.equals(po.getIsCongestion()) && Constants.SystemParam.NO.equals(po.getIsUnbalance())) {
                continue;
            }

            if (Constants.SystemParam.YES.equals(po.getIsSpillover())) {         // 溢出
                status = CrossStatusEnum.SPILLOVER.getCode();
            } else if (Constants.SystemParam.YES.equals(po.getIsCongestion())) { // 拥堵
                status = CrossStatusEnum.CONGESTION.getCode();
            } else if (Constants.SystemParam.YES.equals(po.getIsUnbalance())) {  // 失衡
                status = CrossStatusEnum.UNBALANCE.getCode();
            }
            dto = new CrossDataRealtimePO();
            BeanUtils.copyProperties(po, dto);
            dto.setCrossId(po.getCrossId());
            dto.setStatus(status);
            dtoList.add(dto);
        }
        return dtoList;
    }

    /**
     * 获取路口实时监测数据,获取数据中心存入到数据库中的数据
     * （暂定从数据中心kafka获取）
     *
     * @return
     */
    public List<CrossDataRealtimePO> listCrossDataRealtime() {
        // todo 测试数据
        LambdaQueryWrapper<CrossDataRealtimePO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CrossDataRealtimePO::getCrossId, "14Q0S09IQB0")
                .or().eq(CrossDataRealtimePO::getCrossId, "14Q1409IRF0")
                .or().eq(CrossDataRealtimePO::getCrossId, "14PU609IQA0")
                .or().eq(CrossDataRealtimePO::getCrossId, "14Q1509ISH0")
                .or().eq(CrossDataRealtimePO::getCrossId, "14Q1S09IR70")
                .or().eq(CrossDataRealtimePO::getCrossId, "14Q0O09INT0");
        List<CrossDataRealtimePO> crossDataRealtimePOS = crossDataRealtimeMapper.selectList(queryWrapper);
        return crossDataRealtimePOS;
    }

}