package net.wanji.opt.vo;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import net.wanji.databus.config.DoubleToTwoDecimalPlacesSerializer;

import java.util.List;

/**
 * @author duanruiming
 * @date 2024/11/19 10:48
 * @description 绿波带时序图
 */
@Data
@ApiModel(value = "GreenBeltInfoVO", description = "绿波时序图返回实体")
@JsonIgnoreProperties(ignoreUnknown = true)
public class GreenBeltInfoVO {
    @ApiModelProperty("绿波id")
    private Integer greenId;
    @ApiModelProperty("绿波带长度")
    private Double length;
    @ApiModelProperty("绿波带周期")
    private Integer cycle;
    @ApiModelProperty("绿波带下发时间")
    private String controlTime;
    @ApiModelProperty("策略执行时长")
    private Integer controlDuration;
    @ApiModelProperty("执行策略（1=单条, 2=垂直，3=平行）")
    private Integer type;
    @ApiModelProperty("是否动态绿波（1=是，0=否）")
    private Integer dynamic;
    @ApiModelProperty("-1=stop by error, 0=no control, 1=control")
    private Integer controlMethod;

    private List<DirGreenDetail> dirGreenDetails;

    @Data
    public static class DirGreenDetail {
        @ApiModelProperty("0-反向 1-正向 2-双向")
        private Integer dirType;
        @ApiModelProperty("绿波方向")
        private String dir;
        @ApiModelProperty("最大速度")
        @JsonSerialize(using = DoubleToTwoDecimalPlacesSerializer.class)
        private Double maxSpeed;
        @ApiModelProperty("最小速度")
        @JsonSerialize(using = DoubleToTwoDecimalPlacesSerializer.class)
        private Double minSpeed;
        @ApiModelProperty("绿波带宽")
        private Double greenWidthTime;
        @ApiModelProperty("绿波路口详情")
        private List<CrossGreenDetail> crossGreenDetailList;
    }

    @Data
    public static class CrossGreenDetail {
        @ApiModelProperty("路口")
        private String crossId;
        @ApiModelProperty("相位开始时间")
        private Double phaseStartTime;
        @ApiModelProperty("相位结束时间")
        private Double phaseEndTime;
        @ApiModelProperty("绿波带开始时间")
        private Double greenStartTime;
        @ApiModelProperty("绿波速度")
        @JsonSerialize(using = DoubleToTwoDecimalPlacesSerializer.class)
        private Double speed;
        @ApiModelProperty("相位差")
        private Double offset;
        @ApiModelProperty("行程时间")
        private Double travelTime;
        @ApiModelProperty("到下一个路口距离")
        @JsonSerialize(using = DoubleToTwoDecimalPlacesSerializer.class)
        private Double distance;
    }

}
